@@ -159,8 +159,77 @@ class ProgressionDialog(QtGui.QDialog):
 		for x in chords.chord_shorthand.keys():
 			self.ui.chordsuffix.addItem(x)
 
-		for x in ["1/4", "1/2", "3/4", "1", "2", "4"]:
+		for x in ["0.25", "0.5", "0.75", "1", "2", "3", "4"]:
 			self.ui.bars.addItem(x)
+		self.ui.bars.setCurrentIndex(3)
+
+		self.connect(self.ui.addchord,
+			QtCore.SIGNAL("clicked()"),
+			self.add_chord)
+
+		self.connect(self.ui.addprogression,
+			QtCore.SIGNAL("clicked()"),
+			self.add_progression)
+
+		self.connect(self.ui.buttonBox,
+			QtCore.SIGNAL("accepted()"),
+			lambda: self.save_progression())
+
+	def add_chord(self):
+		chord = str(self.ui.accidentals.currentText())
+		chord += str(self.ui.romannumeral.currentText())
+		chord += str(self.ui.chordsuffix.currentText())
+		duration = str(self.ui.bars.currentText())
+		self.ui.progression.addItem("%s %s" % (duration, chord))
+		self.ui.accidentals.setCurrentIndex(0)
+		self.ui.romannumeral.setCurrentIndex(0)
+		self.ui.chordsuffix.setCurrentIndex(0)
+		self.ui.bars.setCurrentIndex(3)
+
+	def add_progression(self):
+		res=""
+		for x in range(self.ui.progression.count()):
+			t = self.ui.progression.item(x).text()
+			parts = t.split(" ")
+			if parts[0] == "1":
+				text = parts[1]
+			else:
+				text = "%s%s" % (parts[0], parts[1])
+			res += "%s-" % text
+		if res != "":
+			r = str(self.ui.repeat.value())
+			self.ui.progressionsequence.addItem(r + " " + res[:-1])
+
+	def load_progression(self, prog_str):
+		p = str(prog_str)
+		parts = p.split(" ")
+		if len(parts) == 0:
+			return 
+
+		self.ui.progressionname.setText(parts[0])
+		if len(parts) > 3:
+			params = parts[2:-1]
+			for x in params:
+				r = x.split("*")
+				if len(r) == 2:
+					self.ui.progressionsequence.addItem("%d %s" % (int(r[0]) - 1, r[1]))
+				if len(r) == 1 and x != "":
+					self.ui.progressionsequence.addItem("0 %s" % r[0])
+
+	def save_progression(self):
+		res = ""
+		name = str(self.ui.progressionname.text())
+		if name == "":
+			return
+		
+		res += name + " { "
+		for x in range(self.ui.progressionsequence.count()):
+			t = self.ui.progressionsequence.item(x).text()
+			parts = t.split(" ")
+			res += "%d*%s " % (int(parts[0]) + 1, parts[1])
+		res += " }"
+
+		self.item.setText(res)
 
 class BlockDialog(QtGui.QDialog):
 
@@ -339,6 +408,7 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		if cur is None:
 			return
 		p = ProgressionDialog(self.ui.progressions.currentItem())
+		p.load_progression(cur.text())
 		p.show()
 		p.exec_()
 
